#ifndef _OCAMERA_REMOTE_H_
#define _OCAMERA_REMOTE_H_

#ifdef WIN32
  #ifdef oremotecamera_EXPORTS
    #define OREMOTECAMERA_EXP __declspec(dllexport)
  #else
    #define OREMOTECAMERA_EXP __declspec(dllimport)
  #endif
#else
  #define OREMOTECAMERA_EXP
#endif

// from OCamera.h

/**
 * \file OCamera.h
 * \brief Interface definition to camera driver.\n
 *        OCamera is an abstract interface with factory.
 * \author Thomas Berndorfer, Rene Smodic
 * \date 2009/06/10
 */

#ifndef __OCAMERA_H__
#define __OCAMERA_H__


/// properties of an image fetched from the camera
struct OCameraImage
{
  unsigned char *pData;   ///< pointer to image data, data can be read and written
  long nSeqNr;            ///< sequence number of image, can be used to detect lost images
  long nTimeSec;          ///< image time stamp, number of elapsed seconds since 00:00:00, 1.1.1970
  long nTimeMicroSec;     ///< image time stamp, number of microseconds elapsed in the current second
  long nSize;             ///< size of image data in bytes
  long nStatus;           ///< additional status flags for this image
  long nTrigCount;        ///< counted number of pixels from FPGA-based trigger unit
};

//****************************************************************************
//                                                                     OCamera
//****************************************************************************
/**
 *  Basic class to access image acquisition functions.
 *  \par Example Usage
 *         -# create interface with CreateCameraInterface()
 *         -# open it with Open()
 *            (now the file descriptor of the camera is VALID)
 *         -# work with the camera using any other operation
 *            i.e. use
 *              - SetCameraWindow(), SetFrequency()
 *                SetShutterTime(), SetIlluminationMode(), ...
 *              - GetImage(), ReleaseImage()
 *              - GetDriverFileDescriptor()
 *              - ...
 *         -# after work close interface with Close()
 *            (now the file descriptor of the camera is INVALID)
 *         -# if interface is no longer needed call Destroy()
 */
class OREMOTECAMERA_EXP OCamera
{
 public:

  // enums

  /// sensor type of the camera
  typedef enum
    {
      CAM_TYPE_MT9V403   = 3,    ///< Micron MT9V403 CMOS sensor, for VGA cameras
      CAM_TYPE_GENERIC   = 4,    ///< Camera Link camera without serial config line
      CAM_TYPE_IBIS1300  = 5,    ///< FillFactory IBIS5-A/B-1300 1280x1024 pixel sensor
      CAM_TYPE_MT9V022   = 6,    ///< Micron MT9V022 CMOS sensor, for 16:9 VGA camera (752x480)
      CAM_TYPE_MT9V023   = 7     ///< Micron MT9V023 CMOS sensor, for 16:9 VGA camera (752x480)
    }
  eCameraType;

  /// acquisition mode of the camera
  typedef enum
    {
      ACQ_MODE_STOP                = 1,   ///< stop camera activities
      ACQ_MODE_SINGLE_SHOT         = 2,   ///< take single shots only
      ACQ_MODE_FREE_RUN            = 3,   ///< camera is free running
      ACQ_MODE_SELECT_SHOT         = 4,   ///< take single frames from free running camera
      ACQ_MODE_FREE_RUN_SLAVE      = 8,   ///< sensor slave mode, free running
      ACQ_MODE_FREE_RUN_SLAVE_STOP = 11   ///< stop free running slave mode
    }
  eAcqMode;

  /// trigger source of the camera
  typedef enum
    {
      TRIG_SRC_SOFTWARE        = 1,  ///< a call to GetImage triggers acquisition of an image
      TRIG_SRC_EXTERNAL_E0     = 2,  ///< trigger by external digital hardware input signal I0
      TRIG_SRC_EXTERNAL_E1     = 3   ///< trigger by external digital hardware input signal I1
    }
  eTrigSource;

  /// trigger polarity of the camera's trigger
  typedef enum
    {
      TRIG_POL_ACTIVE_HIGH  = 1,  ///< trigger signal is active high, used for TRIG_SRC_EXTERNAL_xx
      TRIG_POL_ACTIVE_LOW   = 2   ///< trigger signal is active low, used for TRIG_SRC_EXTERNAL_xx
    }
  eTrigPolarity;

  /// mode of the internal circular high power LED illumination (available for cameras with internal LED light)
  typedef enum
    {
      ILL_MODE_OFF     = 1,  ///< turn the internal camera illumination off
      ILL_MODE_NORMAL  = 2,  ///< operate LED illumination with nominal current (continuous operation possible)
      ILL_MODE_PULSED  = 3   ///< pulsed LED operation mode (higher brightness but limited to 1:10 duty cycle)
    }
  eIllMode;

  /// mode of an external illumination trigger
  typedef enum
    {
      ILL_MODE_EXT_OFF         = 0,  ///< turn off external light trigger
      ILL_MODE_EXT_ON_POSITIVE = 1,  ///< turn on external light trigger (active high)
      ILL_MODE_EXT_ON_NEGATIVE = 2   ///< turn on external light trigger (active low)
    }
  eIllModeExt;

  /** Pixel frequency values for the camera sensor. Adjustment of the
      pixel clock is not supported on every image sensor. For
      CAM_TYPE_MT9V403 pixel clocks from 20 to 60MHz are supported.
      CAM_TYPE_IBIS1300 uses a fixed frequency of 40MHz.
      CAM_TYPE_MT9V022 and CAM_TYPE_MT9V023 use a fixed frequency
      of 26,6MHz. */
  typedef enum
    {
      FREQU_20MHz = 0,              ///< 20MHz pixel clock
      FREQU_40MHz = 1,              ///< 40MHz pixel clock
      FREQU_50MHz = 2,              ///< 50MHz pixel clock
      FREQU_60MHz = 3               ///< 60MHz pixel clock
    }
  eFrequency;

  /// JPEG Codec quantization level
  typedef enum
    {
      JPEG_QUALITY_LOW       = 0,   ///< low quality, best compression rate
      JPEG_QUALITY_MEDIUM    = 1,   ///< medium quality, very good compression rate
      JPEG_QUALITY_GOOD      = 2,   ///< good quality (standard JPEG quality), good compression rate
      JPEG_QUALITY_EXCELLENT = 3    ///< highest quality but moderate compression rate
    }
  eJPEGQuality;

  /// Laser Profiling point selection priority
  typedef enum
    {
      PRIO_FIRST_POINT   = 0,    ///< select the first matching point
      PRIO_LAST_POINT    = 1,    ///< select the last matching point
      PRIO_LARGEST_FIRST = 2,    /**< select the point with the highest intensity
                                     (if several points have the same maximum intensity
                                     select the one the occurs first in the image) */
      PRIO_LARGEST_LAST  = 3     /**< select the point with the highest intensity
                                     (if several points have the same maximum intensity
                                     select the one that occurs last in the image) */
    }
  ePointPriority;

  /// Laser Profiling evaluation direction
  typedef enum
    {
      DIRECTION_TOP_DOWN  = 0,   ///< output offsets with respect to the top line (top line is zero point)
      DIRECTION_BOTTOM_UP = 1    ///< output offsets with respect to the bottom line (bottom line is zero point)
    }
  eProfileDirection;

  /// Laser Profiling output format selection
  typedef enum
    {
      LASER_OUTPUT_16_4   = 0x04,   ///< selects 16 bit output format including 4 bit subpixel resolution
      LASER_OUTPUT_8_0    = 0x10,   ///< reserved, do not use!
      LASER_OUTPUT_8_1    = 0x11,   ///< reserved, do not use!
      LASER_OUTPUT_8_2    = 0x12,   ///< reserved, do not use!
      LASER_OUTPUT_8_3    = 0x13,   ///< reserved, do not use!
      LASER_OUTPUT_8_4    = 0x14    ///< reserved, do not use!
    }
  eLaserOutputFormat;

  /// Available 3x3 filters
  typedef enum
    {
      FILTER_OFF              = 0,  ///< no filter
      FILTER_BINARIZE_ONLY    = 1,  ///< no filter, allows raw-data binarization
      FILTER_MEDIAN           = 2,  ///< Median filter
      FILTER_MEAN             = 3,  ///< Mean filter
      FILTER_SNN              = 4,  ///< SNN filter
      FILTER_SOBEL            = 5,  ///< Sobel filter
      FILTER_RESERVED_1       = 6,  ///< do not use!
      FILTER_RESERVED_2       = 7,  ///< do not use!
      FILTER_RESERVED_3       = 8   ///< do not use!
    }
  eFilterType;

  /// RGB or HSV(I) color component selection (available only for color sensors)
  typedef enum
    {
      BAYER_OUTPUT_RAW = 0,  ///< raw bayer pattern
      BAYER_OUTPUT_H   = 1,  ///< hue
      BAYER_OUTPUT_S   = 2,  ///< satturation
      BAYER_OUTPUT_V   = 3,  ///< value
      BAYER_OUTPUT_I   = 4,  ///< intensity
      BAYER_OUTPUT_R   = 5,  ///< red
      BAYER_OUTPUT_G   = 6,  ///< green
      BAYER_OUTPUT_B   = 7   ///< blue
    }
  eBayer2XType;

  ///  FPGA bayer pattern selection (only used for color sensors)
  typedef enum
    {
      BAYER_PATTERN_GRGR = 1,    ///< bayer data start with green-red-green-red color pixels
      BAYER_PATTERN_BGBG = 2,    ///< bayer data start with blue-green-blue-green color pixels
      BAYER_PATTERN_RGRG = 3,    ///< bayer data start with red-green-red-green color pixels
      BAYER_PATTERN_GBGB = 4     ///< bayer data start with green-blue-green-blue color pixels
    }
  eBayerPattern;

  /// FPGA-applet types
  typedef enum
    {
      APPLET_IN16_OUT32_INT = 0, ///< standard FPGA applet type. Always use this.
      APPLET_IN8_OUT32_INT  = 1, ///< reserved, do not use!
      APPLET_IN16_OUT16     = 2, ///< reserved, do not use!
      APPLET_IN8_OUT16      = 3  ///< reserved, do not use!
    }
  eAppletType;
  
  /// FPGA image correction types
  typedef enum
    {
      CORRECTION_REPLACE    = 0,  ///< replace sensor data with data stored in FPGA-Image-RAM
      CORRECTION_MASK_AND   = 1,  ///< uses bitwise AND-mask
      CORRECTION_MASK_OR    = 2,  ///< uses bitwise OR-mask
      CORRECTION_RESERVED_1 = 3   ///< reserved, do not use!
    }
  eCorrectionType;

  /// FPGA data splitting types
  typedef enum
    {
      SPLIT_OFF        = 0,  ///< no data splitting enabled
      SPLIT_RAW        = 1,  ///< the raw-sensor image is stored in FPGA-Image-RAM and transferd to the CPU
      SPLIT_PROCESSED  = 2,  ///< reserved, do not use!
    }
  eSplitType;

  /// Filter parameter structure type
  typedef struct sFilterParam
  {
    unsigned long nNumParam; ///< number of parameters to be set
    unsigned long nParam1;   ///< first filter parameter
    unsigned long nParam2;   ///< second filter parameter
  } sFilterParamType;

  /// Binarization parameter structure type
  typedef struct sBinarizationParam
  {
    bool bInvert;                ///< inverts output of the binarization
    unsigned char nNumThreshold; ///< number of threshold values; must always be "2"
    unsigned char nThreshold1;   ///< lower threshold value
    unsigned char nThreshold2;   ///< upper threshold value
  } sBinarizationParamType;


  /// Sensor calibration parameter structure;
  typedef struct sCalibrateSensorParam
  {
    unsigned long nNumParam;
    unsigned long nParam1;
    unsigned long nParam2;
  } sCalibrateSensorParamType;


  /**
   * \name Object Creation/Deletion
   */
  ///@{
  /** Create the image acquisition interface.
   *  Call this method to obtain a pointer to the image acquisition interface through which
   *  all image acquisition functionality can be accessed. The interface
   *  is created specifically for a certain image sensor type. The abstract interface remains
   *  the same for each sensor but the implementation behind is different.
   *  \param eCam specifies the type of sensor mounted in your camera\n
   *             <B>ATTENTION:</B> If you use the wrong sensortype CAM_TYPE_MT9V023 and have
   *             a CAM_TYPE_MT9V022 inside your camera the interface will be created anyway.
   *             The wrong sensortype can only be detected after opening the camera interface
   *             with the function #Open receiving an error <I>-ENODEV</I>. \n
   *             If you are unsure which sensor (MT9V022 or MT9V023) is inside your camera
   *             follow these steps:
   *               - create the interface with CreateCameraInterface(OCamera::CAM_TYPE_MT9V023)
   *               - open the interface with #Open
   *               - if returnvalue of #Open equals <I>-ENODEV</I>
   *                  - destroy the wrong interface with #Destroy
   *                  - create the interface with CreateCameraInterface(OCamera::CAM_TYPE_MT9V022)
   *                  - open the interface again with #Open
   */
  // object creation/deletion
  static OCamera *CreateCameraInterface(eCameraType eCam);
   /** Removes completely the interface object created with CreateCameraInterface().
   * This should happen after Close() was called and the interface is
   * no longer needed.\n
   * Be careful to <B>DO NOT USE THE INTERFACE POINTER</B> any longer after
   * having called this method!!
   */
  virtual int Destroy() = 0;
  ///@}

  // -----------------------------------------------------------------
  // operations
  // -----------------------------------------------------------------
  /**
   * \name Operations
   */
  ///@{
  // open and close the camera interface
  /** Opens the camera interface <B>BEFORE ANY OTHER</B> method is called.
   *  After creation of the image acquisition interface through CreateCameraInterface()
   *  the interface must be opened, i.e. underlying driver functions get called
   *  to put the image sensor out of reset into an operable state. \n
   *  A call to Open() prepares internal data structures and allocates memory to hold
   *  image data. It is an important to know that image memory is managed
   *  at device driver level, i.e. in the linux kernel space. This is necessary for
   *  free running acquisition modes where images are acquired in the background by the
   *  driver without interaction with the user process. When it is time, the user process
   *  fetches the next pending image from the driver and maps corresponding image memory
   *  to user space (no copy of data necessary). The amount of memory allocated at driver
   *  level is specified as an argument to this method. \n
   *  After opening the interface by this call any other operations from this interface
   *  may be called.
   * \param dFrameBufferSize size of the driver level image memory buffer in bytes\n
   *        <B>ATTENTION:</B> A minimum size of 2*(used resolution) is needed if
   *         dMaxBuffers equals -1
   * \param dMaxBuffers specifies the maximum number of image buffers (i.e. images)
   *        to be allocated in the driver. For dMaxBuffers == -1 the number of buffers
   *        is adjusted automatically to dFrameBufferSize for every call to
   *        SetCameraWindow(), i.e. smaller windows result in a larger number of
   *        image buffers and larger windows result in smaller number of image buffers.
   *        Specifying dMaxBuffers != -1 can limit the number of image buffers allocated
   *        in the driver to the given maximum expecially in cases where the image
   *        size is very small. In such cases overhead allocations in the driver
   *        may result in significant memory consumption otherwise.
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)\n
   *    -ENODEV ... maybe the wrong eCameraType has been used to create the interface
   *                (s.a. function #CreateCameraInterface) => destroy the wrong interface
   *                and create a new one with the correct eCameraType before calling Open again
   */
  virtual int Open(int dFrameBufferSize = 4096*256, int dMaxBuffers = -1) = 0;

  /** Closes the camera interface <B>AFTER</B> all work has been done.
   *  This closes the image acquisition interface and frees memory allocated at driver
   *  level and inside the API. Any connections to device drivers are closed.
   *  After calling this function an external file descriptor received with GetDriverFileDescriptor()
   *  is no longer valid. Keep this in mind and avoid usage of this file descriptor for
   *  select() calls after calling this method.
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int Close() = 0;

  /** Get the next image from this interface.
   *  Images are acquired from the image sensor at device driver level. Any image that is
   *  fully acquired by the driver is put into a list of pending images. Calling this method
   *  immediately returns the next image from the pending list if one is available. If the
   *  pending list is empty the method blocks and waits until the next image is available.
   *  To prevent from program hangs, a maximum amount of time to wait (timeout) can be set by
   *  a call to SetReadTimeout().
   *
   *  <P>Images returned by this method are kept on an extra read list which prevents images on
   *  that list to be used for subsequent acquisitions, i.e. an image returned by this method
   *  is left untouched until it is passed back to the driver by a call to ReleaseImage().</P>
   * \param pImage pointer to an #OCameraImage object
   * \return
   *          0 ... on success \n
   *        -62 ... timeout occurred\n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int GetImage(OCameraImage *pImage) = 0;

  /** Releases the image and returns image memory space to the framebuffer.
   *  As image memory is managed by the device driver, images returned by GetImage() are
   *  under exclusive access by the user until ReleaseImage() is called on these images.
   *  This allocate/free mechanism prevents images from being overwritten while currently
   *  processed.
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int ReleaseImage(OCameraImage *pImage) = 0;

  /** Gets the maximum number of images (immediate sequenced  GetImage() calls).
   *  The return value equals the number of images that can be buffered in the driver.
   *  This number approximately equals the totally allocated buffer space given by Open()
   *  divided by the size of an image. But this is an estimation due to the memory allocation
   *  granularity in the kernel and block size effects when managing image buffers (images
   *  must completely fit in an allocated block).
   * \return maximum number of images, that can be buffered in the driver
   */
  virtual int GetMaxImageCount() = 0;

  /** Get the number of images currently pending.
   *  The return value equals the number of images currently residing in the list
   *  of pending images, i.e. images that are successfully acquired but not read by the user
   *  through calls to GetImage().\n
   *  In other words this method returns the number of consecutive calls to GetImage()
   *  without a blocking wait.
   * \return number of images currently pending
   */
  virtual int GetImageCount() = 0;

  /** Set the mode of image acquisition.
   *  This is an essential method and must be called <B>PRIOR</B>
   *  to getting any image from the camera. Normally this is the last method called
   *  during initialisation of the image acquisition interface before fetching the
   *  very first image.
   *
   *  <P>This camera API supports a large set of several different acquisition modes
   *  with a completely different behaviour of the individual modes:</P>
   *
   *  <P>ACQ_MODE_STOP enters a stop state where no image acquisition takes place.
   *  It makes sense to enter this mode to unload the acquisition engine from unnecessarily
   *  performed work. This unloads the system memory bus (no DMA) and saves power. </P>
   *  <P>In ACQ_MODE_SINGLE_SHOT each image must be triggered explicitely either per software
   *  through the call to GetImage() or an external hardware trigger input signal. Only one
   *  image is acquired for one single trigger. This mode is typically used for image
   *  processing applications where external devices (like PLCs) signal when an image from
   *  a manufactured part should be taken and evaluated. Another typical situation is a
   *  cyclically running applications that take images, process them and acquire the next
   *  images in an infinite loop at the maximum possible speed. </P>
   *
   *  <P>ACQ_MODE_FREE_RUN operates the image sensor in a continuously running mode where
   *  one image is taken immediately after the other. The resulting frame rate depends on
   *  the size of the image (transfer time from sensor array to memory) and the exposure
   *  time. Image acquisition immediately starts after calling SetAcquisitionMode() with
   *  this mode as the argument. Images are buffered by the driver and the application must
   *  keep track of the resulting frame rate. If processing time in the application is too
   *  long the image buffer may overflow after a while resulting in images to be lost.</P>
   *
   *  <P>ACQ_MODE_SELECT_SHOT is similar to ACQ_MODE_FREE_RUN and operates the image sensor
   *  in the same mode. The important difference is that the image data stream from the sensor is
   *  normally blocked resulting in all images to be lost except when GetImage() is called.
   *  In other words GetImage() simply fetches the next available image in the stream of
   *  images from the continuously running sensor. In this mode the image buffer memory
   *  can not overrun and the time it takes for GetImage() to return is less in average
   *  than for ACQ_MODE_SINGLE_SHOT where it takes at least the exposure time plus the read
   *  out time for GetImage() to return.</P>
   *
   *  <P>The most professional mode of operation is ACQ_MODE_FREE_RUN_SLAVE. In this mode the
   *  cameras FPGA completely takes control of what is going on during image acquisition
   *  (the sensor is in slave mode). ACQ_MODE_FREE_RUN_SLAVE continuously acquires images
   *  with a certain adjustable exact frame rate. It is the only mode for the MT9V403 sensor
   *  where image data read out takes place concurrently with sensor exposure resulting in
   *  a maximum speed in terms of frames per second. ACQ_MODE_FREE_RUN_SLAVE is typically
   *  used for applications where a fixed frame rate is necessary (e.g. scanning applications)
   *  and adjustment of exposure time doesn't influence the frame rate. This would be different
   *  in ACQ_MODE_FREE_RUN. </P>
   *
   *  <P>ACQ_MODE_FREE_RUN_SLAVE_STOP is a mode to temporarily stop the fixed frame rate
   *  acquisition of the ACQ_MODE_FREE_RUN_SLAVE mode, e.g. to enter a calculation phase
   *  before the next sequence of images is taken again. ACQ_MODE_FREE_RUN_SLAVE_STOP
   *  enters a stop state with almost no delay in comparison to ACQ_MODE_STOP which needs
   *  at least 50ms to take place.\n
   *  ATTENTION if you have a large number of (small) images => call of function Close()
   *  may take same time (up to 700msec!!)</P>
   * \param eAcq desired acquisition mode of the camera
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetAcquisitionMode(eAcqMode eAcq) = 0;

  /** Specify the mode and source of the trigger.
   *  A trigger to acquire an image can be generated by software,
   *  FPGA and external hardware input signals
   * \param eTrig source of the trigger
   * \param ePol polarity of the trigger
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetTriggerSource(eTrigSource eTrig,
                               eTrigPolarity ePol = TRIG_POL_ACTIVE_HIGH) = 0;

  /** Set the internal illumination mode of the camera (cameras with internal LED light).
   * \param eIll mode of the internal circular high power LED illumination
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetIlluminationMode(eIllMode eIll) = 0;

  /** Set the external illumination trigger mode of the camera.
   * \param eIll mode of the external illumination
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetIlluminationModeExternal(eIllModeExt eIll) = 0;

  /** Specify position and duration of the pulse which switches on the
   *  illumination light source.
   * \param dDelay is the delay of the illumination pulse with resp. to trigger
   *               in microseconds (1E-6)
   * \param dPeriod is the duration of the illumination pulse in microseconds (1E-6)
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetIlluminationTiming(int dDelay, int dPeriod) = 0;

  /** Permanently switch on/off internal light (only for cameras with internal LED light).
   * \param bOn
   *         - TRUE  ... switch on internal light
   *         - FALSE ... switch off internal light
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SwitchIllumination(bool bOn) = 0;

  /** Setup the shutter time in microseconds (1E-6).
   *  Depending on the acquisition mode the effective shutter time
   *  may be different from the given value (e.g. if the camera
   *  has predefined discrete exposure times in free run mode)
   * \param dShutter range from 0 to xxx.xxx microseconds
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetShutterTime(int dShutter) = 0;

  /** Get the actual value of shutter time in microseconds (1E-6).
   *  Depending on the type of image sensor shutter time can be
   *  configured in a certain granularity (most often in multiples
   *  of row times). Hence, not all values specified in SetShutterTime()
   *  can be actually configured on the sensor. This method returns
   *  the actually configured value of shutter time in microseconds.
   * \return
   *          the currently set shutter time in microseconds \n
   */
  virtual int GetShutterTime(void) = 0;

  /** Set the position and size of the camera sensor window.
   *  Have a look at the camera data sheet first to specify
   *  valid coordinates for this function. \n
   *  Because of the way how DMA transfers from the sensor are made
   *  it is required that dWidth*dHeight is a multiple of 32bytes!
   * \param dX0 x-value of left upper corner
   * \param dY0 y-value of left upper corner
   * \param dWidth width in x direction
   * \param dHeight height in y direction
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetCameraWindow(int dX0, int dY0, int dWidth, int dHeight) = 0;

  /** Set the camera's frame rate (the unit is "frames per second").
   *  This function may only be called in situations where the
   *  frame rate can be adjusted independently from the shutter
   *  time else this function has no effect. \n
   *  SetCameraWindow() and SetShutterTime() must be called <B>PRIOR</B> to this method
   *  to properly adjust camera timing!!
   * \param dFps frame rate in "frames per second"
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetFrameRate(int dFps) = 0;

  /** Set the camera's frame rate (the unit is "frames per second", float version).
   *  This function may only be called in situations where the
   *  frame rate can be adjusted independently from the shutter
   *  time else this function has no effect. \n
   *  SetCameraWindow() and SetShutterTime() must be called <B>PRIOR</B> to this method
   *  to properly adjust camera timing!!
   * \param fFps floating point frame rate in "frames per second"\n
   *             <B>DO NOT</B> use frame rates below 1.0fps
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetFrameRate(float fFps) = 0;

  /** Delay the trigger pulse for a specified amount of microseconds (1E-6).
   *  The delay is inserted between trigger event and actual trigger impulse to the camera.
   * \param dDelay delay of the trigger pulse in microseconds (1E-6)
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetTriggerDelay(int dDelay) = 0;

  /** Set the camera sensor gain as a decimal factor.
   *  Make sure to specify valid gain values here.
   * \param dFactor gain value for the camera sensor \n
   *         MT9V403 range from 1 to 18 \n
   *         IBIS1300 range from 1 to 16 \n
   *         MT9V022 range from 1 to 25 \n
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetGain(int dFactor) = 0;

  /** Switches the API/driver into color mode.
   *  This setting has to be made corresponding to the installed FPGA design -
   *  so if you have a color camera (using a color FPGA-Design), call this function
   *  when your application starts up.
   *  By default the camera driver starts up in greyscale mode - only a color camera has to 
   *  be switched explicitly into the correct operation mode before its special FPGA features can be used.
   *  ATTENTION: Operating a color camera in greyscale mode (and vice versa) can and will
   *  lead to unexpected results!
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int InitColor() = 0;

  /** Set the timeout value to wait during a GetImage() operation.
   *  This may prevent from hanging if no image could be
   *  acquired (possible cause: no camera cable or no trigger signal etc.)
   *  Without calling this function a default value of 2s is used as
   *  timeout period. If you whish to wait for infinite time pass a
   *  value of -1 to this function.
   * \param dTimeout timeout value in microseconds (1E-6s)\n
   *                 use a value of -1 to wait for infinite time
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetReadTimeout(int dTimeout) = 0;

  /** Retrieve file descriptor to perform select/poll on the camera interface driver.
   *  ATTENTION: You cannot use FD_SET (part of the select function usage) with a negative
   *  file descriptor.
   * \return file descriptor on the camera interface\n
   *         < 0 if camera interface is not open (s.a. Open())
   */
  virtual int GetDriverFileDescriptor() = 0;

  /** Set circular image overwrite mode for situations, where the
   *  camera runs free until a trigger event occurs.
   * \param bOverwrite
   *         - TRUE  ... enable circular overwrite
   *         - FALSE ... disable circular overwrite
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetCircularOverwrite(bool bOverwrite) = 0;

  /** Enables or disables the JPEG coder, specify desired quanitsation (image quality) and
   *  allows enabling of the hardware bytestuffing used in the JFIF file format.
   *  On color cameras this call furthermore allows to set the Bayer-Pattern starting value and
   *  enforcement of greyscale compression (this parameters are ignored on greyscale cameras).
   * \param bEnable
   *         - TRUE  ... enable circular overwrite
   *         - FALSE ... disable circular overwrite
   * \param eQuality image quality of JPEG compression
   * \param bByteStuffEnable
   *         - TRUE  ... enables hardware byte-stuffing
   *         - FALSE ... disable hardware byte-stuffing
   * \param ePattern sets the start bayer pattern for color hardware JPEG encoding, only effective for color sensor
   * \param bForceGreyscale allows to encode greyscale images on color hardware JPEG encoder
   *         - TRUE  ... encode as greyscale image
   *         - FALSE ... encode as color image
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetJPEG(bool bEnable, eJPEGQuality eQuality = JPEG_QUALITY_MEDIUM,
                      bool bByteStuffEnable = false, eBayerPattern ePattern = BAYER_PATTERN_GRGR,
                      bool bForceGreyscale = false) = 0;

  /** Set camera clock frequency.
   * Adjust the system clock frequency of the image sensor. This must be done immediately
   * after calling Open() to allow for a short settling time before making any settings
   * like SetShutterTime() or SetCameraWindow().\n
   * Keep in mind that changing the clock frequency is not supported by all image sensor variants: \n
   * MT9v403:  20MHz, 40MHz, 50MHz, 60MHz \n
   * IBIS1300: fixed at 40MHz \n
   * MT9v022:  fixed at 26,6MHz \n
   * MT9v023:  fixed at 26,6MHz \n
   * \param eFrequ camera clock frequency
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetFrequency(eFrequency eFrequ) = 0;

  /** Get an image to a buffer supplied by the user.
   *  The image data is directly written to the buffer from the DMA controller.
   *  This function can be used in snapshot modes <B>ONLY</B>
   * \param pImage pointer to an #OCameraImage object
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int GetImage2Buffer(OCameraImage *pImage) = 0;

  /** Transfer a block of memory from the image SDRAM to a buffer supplied by the user.
   *  This SDRAM is only accessible via the FPGA. Transfers are made by DMA.
   *  This function can be used in snapshot modes <B>ONLY</B>
   *  A total number of ulLen bytes is transferred from the given image SDRAM address ulAddr.
   *  pDest must point to a 32-byte aligned buffer having a length of at least ulLen bytes.
   *  The amount of transferred data (ulLen) must always be a multiple of 32 Bytes and ulAddr must be an even byte-address.
   * \param pDest destination (user-space) memory buffer, which will be written to \n
   *             length of buffer has to be >= ulLen
   * \param ulAddr start address in image SDRAM
   * \param ulLen number of bytes to read from image SDRAM
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int ReadImageRAM(void *pDest, unsigned long ulAddr, unsigned long ulLen) = 0;

	/** Transfer a block of memory (out of a user buffer) to the image SDRAM.
   *  This SDRAM is only accessible via the FPGA. Transfers are made by DMA.
   *  This function can be used in snapshot modes <B>ONLY</B>
   *  A total number of ulLen bytes is transferred to the given image SDRAM address ulAddr.
   *  pSrc must point to a 32-byte alligned buffer having a length of at least ulLen bytes.
   *  The ammount of transfered (ulLen) data must always be a multiple of 32 Bytes and ulAddr must be an even byte-address.
   * \param pSrc source (user-space) memory buffer, which will be read from \n
   *             length of buffer has to be >= ulLen
   * \param ulAddr start address in image SDRAM for write operation
   * \param ulLen number of bytes to write to image SDRAM
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int WriteImageRAM(void *pSrc, unsigned long ulAddr, unsigned long ulLen) = 0;

  /** This function is used to read a defined ammount of data (for example a raw-image) from the image SDRAM
   *  via a hardware applet or the hardware JPEG encoder. The applet/JPEG coder outputs an reduced ammount
   *  of data that is transferred to the CPU's memory.
   *  Transfers are made by DMA. The end of a transfer is signaled to the PXA with a special interrupt
   *  and the total number of transfered bytes is returned.
   *  This function can be used in snapshot modes <B>ONLY</B>
   *  pDest must point to a 32-bit alligned buffer having a length of at least ulLen bytes.
   *  The ammount of requested data (ulLen) must always be a multiple of 32 Bytes and start at an even address (ulAddr).
   * \param pDest destination (user-space) memory buffer, which will be written to \n
   *             length of buffer has to be >= ulLen
   * \param ulAddr start address of RAW image in image SDRAM
   * \param ulLen  number of bytes to read from image SDRAM
   * \param *ulRetLen  ammount of transferred data in bytes
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int ReadImageRAMReducedData(void *pDest, unsigned long ulAddr, unsigned long ulLen,
                                      unsigned long *ulRetLen) = 0;

  /** Transfers a block of memory in the image SDRAM to another position in the image SDRAM.
   *  During the transfer FPGA-filters, binarization, and color component selection can be enabled.
   *  This function can be used in snapshot modes <B>ONLY</B>
   *  Source size and destination size should be set to the same value (except for hardware applet usage).
   *  Enabling JPEG compression with this function is not allowed!
   * \param ulSrcAddr source address in image SDRAM
   * \param ulSrcLen  source size
   * \param ulDestAddr destination address in image SDRAM
   * \param ulDestLen destination size
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int ImageRAMToImageRAM(unsigned long ulSrcAddr, unsigned long ulSrcLen,
                                 unsigned long ulDestAddr, unsigned long ulDestLen) = 0;

  /** Initialize the image RAM.
   *  If your application wants to use the image SDRAM, call this function first.
   *  Executon of any image SDRAM related functions before InitImageRAM() will lead to
   *  unexpected results.
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int InitImageRAM(void) = 0;
  /** Disable the image RAM. When this function is called the image SDRAM will loose all 
   *  stored data!
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int DisableImageRAM(void) = 0;
  /** Take one sensor image and transfer it to the image SDRAM.
   *  Works only with raw images. The hardware JPEG encoder must not be enabled when using this function.
   *  The used start address must be an even byte address
   * \param ulAddr start address in image SDRAM for write operation
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int GetImageToImageRAM(unsigned long ulAddr) = 0;

  /// Diagnose function
  virtual int Diag() = 0;

  /** Reset the sequence counter to zero.
   *  Every image carries a sequence number attached by hardware
   *  which is incremented by one each time an image is acquired.
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
 virtual int ResetSequenceCounter() = 0;

  /** Purges the driver's frame buffer removing any previously acquired images.
   *  Whenever an image acquisition is stopped and later re-enabled
   *  the user may use this function to purge the driver's frame buffer
   *  to remove any previously acquired image up to the point of time
   *  when calling this function.
   *
   *  <P>Note, that any image that was read by GetImage() and that ist NOT
   *  returned by ReleaseImage() will <B>NOT</B> be freed, i.e. the user may
   *  still borrow some images from the driver's frame buffer without getting
   *  them overwritten.</P>
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int PurgeImages() = 0;

  /** Perform sensor calibration for IBIS1300. Adjusts the DAC_RAW and DAC_FINE to obtain
   *  equal behaviour of even and odd colums and overall zero black level.
   *  Resulting values are stored in Param1 and Param2 of the pParam structure as long as it 
   *  is not zero.

   * \param *pParam pointer to the parameter structure
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int CalibrateSensor(sCalibrateSensorParamType *pParam = 0) = 0;


  /** Write sensor specific configuration register.
   *  This method enables the user to access configuration registers of the
   *  imaging sensor for fine tuning purposes. Normally it is not necessary
   *  to use this function. But some image sensors offer special features
   *  the user may want to use. Be especially careful when
   *  changing sensor registers to different values because this may lead to
   *  trouble with image acquisition.
   * \param pName is a string specifying the name of the image sensor register
   * \param nValue specifies the value that should be written to the given register
   * \return
   *          0 ... on success \n
   *    -EINVAL ... invalid register name \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetSensorRegister(const char *pName, unsigned long nValue) = 0;

  /** Read sensor specific configuration register.
   *  This method enables the user to access configuration registers of the
   *  imaging sensor for fine tuning purposes. Normally it is not necessary
   *  to use this function for acquiring images. But some image sensors offer
   *  special features the user may want to use. Note that some sensors do not
   *  offer a possibility to read back its registers.
   * \param pName is a string specifying the name of the image sensor register
   * \param pValue is a pointer to an unsigned 32bit value where the contents of
   *  the specified register is written to.
   * \return
   *          0 ... on success \n
   *     -EPERM ... no read access to registers on this sensor possible \n
   *    -EINVAL ... invalid register name \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int GetSensorRegister(const char *pName, unsigned long *pValue) = 0;

  /** Ask for the names of registers.
   *  This method can be used to ask for individual names of sensor registers
   *  supported for reading and writing.
   * \return A pointer to an array of strings (char * pointers) with a zero
   *  entry marking the end of the array.
   */
  virtual char** GetSensorRegisterNames() = 0;

  /** Trigger an image trigger in ACQ_MODE_SINGLE_SHOT.
   *  This enables the user to perform image acquisition and processing in parallel.
   *  This does <B>NOT</B> wait until the image is ready. The acquisition
   *  is only started (i.e. the sensor is triggered). To get the image
   *  itself, someone has several options:
   *    -# call GetImage() immediately afterwards or some time later which
   *       immediately returns the image if it is already available or may
   *       wait until the image is available
   *    -# call GetImageCount() to ask for the number of images available
   *    -# call select() on the file descriptor returned by
   *       GetDriverFileDescriptor() until the file descriptor gets readable
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int TriggerImage() = 0;

  /** Abort image acquisition previously started by TriggerImage().
   *  This method allows to abort an ongoing image acquisition that has been
   *  previously started by TriggerImage(). This functionality may be very
   *  useful if hardware triggered acquisition is used to prevent from undesired
   *  waits.
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int TriggerImageAbort() = 0;

  /** Setup laser scan profiling.
   *  This method prepares the camera do start with laser profiling, i.e. images
   *  are preprocessed by the FPGA to find a bright (or dark) line in the image.
   *  The position of the line (offset from top/bottom) is output instead of
   *  the image.
   *
   *  <P>One 16 bit output offset has a 12bit integer part and a 4 bit
   *  fractional part to allow for sub pixel accuracy.
   *  The profile can be taken
   *  only horizontally, i.e. for an image of 640x480 pixel you get 640 offset
   *  values. You can specify any position and size of the image taken
   *  from the sensor that produces output data sizes in a multiple of 32 bytes.
   *  (i.e. you can specify the window in multiples of 16 pixel columns as each
   *  column produces a 16bit output value). </P>
   *
   *  <P>Output data is fetched from the FPGA via the normal call to GetImage() with
   *  the 'column wise 16bit offsets as the resulting image'. \n
   *  We recommend to use acquisition mode FREE_RUN_SLAVE to produce profiles
   *  at fixed frame rates. But any other acquisition mode is possible. Keep in mind
   *  that reducing the number of lines in an image increases the highest possible
   *  frame rate but reducing the number of columns does not (but of course the
   *  amount of data will be reduced).\n
   *  Every profile you acquire via the GetImage() call has a resulting size from 2*N+32 bytes. This
   *  includes the 16-bit values for N columns and an additional 32 byte data block. This block is
   *  reserved for future use and should be ignored.\n
   *  Output offsets can be calculated with respect to the top or bottom line
   *  depending on your needs in the application. A value of zero is returned if
   *  no laser line was found at the corresponding position. If a laser line was found
   *  at the top(eDir==DIRECTION_TOP_DOWN)/bottom(eDir==DIRECTION_BOTTOM_UP) row of an
   *  image, an output offset of 1 is returned. \n
   *  Acquiring normal images after laser profiling requires to call SetupLaserProfiling
   *  with the bEnable parameter set to false and a subsequent SetCameraWindow() call
   *  to re-establish the camera window for normal image acquisition.</P>
   *  <P>Note: for the eLaserOutputFormat parameter only LASER_OUTPUT_16_4 can be used,
   *  and the pRefData parameter is ignored!</P>
   * \param bEnable enables or disables the laser line profiling unit
   * \param dX0 x-value of the upper left corner of the acquired images
   * \param dY0 y-value of the upper left corner of the acquired images
   * \param dWidth width of image in x direction (columns), must be a multiple of 16!
   * \param dHeight height of image in y direction (lines), depends on desired frame rate
   * \param ePrio specifies which point to return when processing one image column
   * \param eDir specifies offsets to be calculated with respect to the top or bottom line
   * \param dThreshMin minimum grey level for a pixel to be treated as laser spot pixel
   * \param dThreshMax maximum grey level for a pixel to be treated as laser spot pixel
   * \param dAreaMin minimum grey level sum of consecutive laser spot pixels for a laser spot,
   *        must fit into 14bit, i.e. dAreaMin < 16384
   * \param dAreaMax maximum grey level sum of consecutive laser spot pixels for a laser spot,
   *        must fit into 14bit, i.e. dAreaMax < 16384
   * \param dSubRes Laser Profiling output format (only LASER_OUTPUT_16_4 available)
   * \param pRefData data of laser reference line (ignored for LASER_OUTPUT_16_4)
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetupLaserProfiling(bool bEnable,
                                  int dX0, int dY0, int dWidth, int dHeight,
                                  ePointPriority ePrio, eProfileDirection eDir,
                                  int dThreshMin, int dThreshMax,
                                  int dAreaMin, int dAreaMax, eLaserOutputFormat dSubRes = LASER_OUTPUT_16_4,
                                  unsigned char *pRefData = 0) = 0;

  // Generic Hardware Applet interface
  /** Enable or disable a customer specific FPGA design.
   * This method must be called if you work with Silicon Software Visual Applets
   * or design your own FPGA preprocessing solution for this camera. Customized
   * preprocessing designs (also called hardware applets) offer an
   * additional path for image data to pass through the FPGA (i.e. image data can flow
   * through your preprocessing or not). You can use this method to select between
   * the preprocessing path or the direct pass through without any preprocessing.
   * \param bEnable
   *         - TRUE  ... enable hardware applet preprocessing
   *         - FALSE ... disable hardware applet preprocessing and directly read in
   *                     raw image data
   * \param nMaxSize specifies the maximum amount of data (worst case) that can
   * be delivered by the hardware applet. \n
   * Normally this is the size of an image in all cases where the hardware applet
   * doesn't remove or add any data to an image. If compression is done the
   * resulting size of an image may be significantly lower, whereas a bayer-to-rgb
   * transform triples the amount of data. In any way keep in mind that nMaxSize
   * specifies nothing else than the size of a DMA buffer to allocate.
   * The resulting actual size of an image after preprocessing is always
   * returned in the nSize field of the OCameraImage struct. Specifying nMaxSize too
   * low results in data being cut off from an image. Specifying nMaxSize too large
   * wastes memory. So you really should set nMaxSize with some care.
   * \param eApplet type of FPGA-Applet. By default use APPLET_IN16_OUT32_INT 
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetHardwareApplet(bool bEnable, unsigned long nMaxSize, eAppletType eApplet = APPLET_IN16_OUT32_INT) = 0;

  // Median filter interface
  /** INFO: This is an legacy function. Please use SetFilter() instead!
   *  Enable or disable the FPGA based 3x3 Median filter.
   *  Median filtering is done in FPGA without any speed loss and without
   *  additional latency during image acquisition, i.e. the median filter
   *  immediately outputs its first line of image data when the first
   *  line of input data arrives on its input. The only drawback of the
   *  applied full streaming algorithm is that median filtered images
   *  appear to be one image line shifted down. \n
   *  Note: Due to limited onchip FPGA memory ressources median filter and
   *  laser line profiling cannot coexist on a -R2 (IBIS1300) camera.
   *  Default FPGA designs support laser line profiling. Please contact
   *  your distributor or camera vendor to obtain a median filter enabled
   *  FPGA design for the IBIS1300 camera.
   *  Note2: Before initiating an ImageRAM data-transfer over the median-filter
   *  make sure to use SetImageRAMPictureSize().
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetMedian(bool bEnable) = 0;

  /** Enable or disable one of the FPGA based filters (Median, Mean, SNN, Sobel)
   *  Before initiating an ImageRAM data-transfer over an filter make sure to use
   *  SetImageRAMPictureSize().
   *  There is also a set of configurable filter parameters available. For exact definitions
   *  of these parameters please refer to the online documentation (chapter. "FPGA Design and FPGA-ImageRAM").
   *  These parameters are by default initialized with meaningfull values.
   *  Note: Due to limited onchip FPGA memory ressources median filter and
   *  laser line profiling cannot coexist on a -R2 (IBIS1300) camera.
   *  Default FPGA designs support laser line profiling. Please contact
   *  your distributor or camera vendor to obtain a median filter enabled
   *  FPGA design for the IBIS1300 camera.
   * \param  eFilter the filter to be switched on. Use FILTER_OFF for disabling
   * \param  pParam  pointer to structure containing filter parameters.

   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetFilter(eFilterType eFilter, sFilterParamType *pParam = 0) = 0;

 /** Enable or disable the FPGA-Binarization. The actual FPGA design supports exactly two threshold values,
   * which must be set in the parameter structure (THRESH_LO,THRESH_HIGH).
   * The result of the binarization is the following for each pixel:
   * 255 when (THRESH_LOW <= x <= THRESH_HIGH) else 0
   * By using the bInvert parameter the output (255/0) is inverted to (0/255)
   * Note: The binarization is executed within the FPGA's filter block. Therefore, if you want to
   * binarize a RAW image (on color camaras a Bayer2X processed image), call SetFilter() with its
   * FILTER_BINARIZE_ONLY parameter before.
   * Note2: When you transfer data over a filter from the Image-RAM, make sure to set
   * the correct image size with SetImageRAMPictureSize() before.
   * \param  bEnable        turns on/off binarization
   * \param  pParam         pointer to binarization paramter structure (invert, thresholds)
   */
  virtual int SetBinarization(bool bEnable, sBinarizationParamType *pParam = 0) = 0;

  /** Used on color cameras to select one of the following color components: red, green, blue;
   *  hue, saturation, value, intensity. For details please refer to the online documentation
   *  chapter "FPGA Design and FPGA-ImageRAM".
   *  Note: if you want to use a FPGA-Filter on color cameras, you also have to use this function and select
   *  a color component.
   * \param eBayer2xOutput  selects color component emmited by Bayer2X unit
   * \param ePattern        sets Bayer-Pattern start value
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetBayer2X(eBayer2XType eBayer2xOutput, eBayerPattern ePattern) = 0;

  /**   Setting up the FPGAs-whitebalancing tables.
   *    Only useable with color cameras.
   *    The FPGA-whitebalancing is based on three lookup-tables for red, green and blue. Each of these tables
   *    contains 256 bytes. All three tables are alligned in one single memory block (768 bytes) in GBR order.
   *    White balancing is always turned on, but by default the lookup tables are initialized linear and
   *    therefore there is no effect. This means for turning off FPGA-whitebalancing you have to write linear
   *    tables to the FPGA. The lookup-table is also reseted when the function is called without the pLut parameter.
   * \param ePattern defines how the sensors Bayer-Pattern starts at the used camera window. 
   * \param pLut  pointer to 768 byte array containing the three lookup tables
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   *     -ENOMEM... if unable to reset lookup tables
   */
  virtual int SetWhiteBalance(eBayerPattern ePattern, unsigned char* pLut = 0) = 0;

  /**   When reading image data out of the ImageRAM this function must be used to setup
   *    the width and heigth of the image before calling ReadImageRAM()/ImageRAMToImageRAM()/etc.
   *    This is necessary because the FPGA does not know about the size of an image stored into the ImageRAM.
   *    The value-pair 0/0 for dWidth/dHeight should be used for general data transfers containing no images.
   *    dWidth and dHight must always be multiples of two.
   * \param dWidth number of columns of the image in ImageRAM
   * \param dHeight number of rows of the image in ImageRAM
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int SetImageRAMPictureSize(int dWidth, int dHeight) = 0;

  // FPGA-based triggering
  /** Setup FPGA based image triggering. The FPGA based trigger unit
   *  counts the number of
   *  pixels having a grey level between a minimum and maximum threshold
   *  inside a rectangular window of the the image. The coordinates
   *  of the window and the values for the thresholds can be adjusted
   *  by this function call. The counted number of pixels is reported
   *  for each acquired image in the associated OCameraImage structure
   *  (see nTrigCount).
   *  Reading this pixel counter value the user can easily decide whether
   *  he wants to discard or further process the image. A typical example
   *  is that someone wants to wait until an object becomes visible
   *  in a certain area of the image.
   *  Note, that the specified trigger window is always relative to the
   *  camera window, i.e. a call to SetCameraWindow() also affects the
   *  position of the trigger window.
   * \param dX0 x-value (relative to position of the camera window,
   *  see SetCameraWindow()) of upper left corner of trigger window
   * \param dY0 y-value (relative to position of te camera window,
   *  see SetCameraWindow()) of upper left corner of trigger window
   * \param dWidth width of trigger window in x direction
   * \param dHeight height of trigger window in y direction
   * \param dThreshMin minimum grey level for a pixel to be counted
   * \param dThreshMax maximum grey level for a pixel to be counted
   */
  virtual int SetTriggerWindow(int dX0, int dY0,
                               int dWidth, int dHeight,
                               int dThreshMin, int dThreshMax) = 0;

  /** This method is used to enable a FPGA preprocessing step where the sensors
   * image data is combined with data previously stored in the FPGAs Image-SDRAM.
   * For each pixel a 8-Bit value is read from the image SDRAM and combined with the pixel
   * in the way defined by the eType parameter.
   * This method for example can be used to replace the sensors image with a testimage
   * stored in the image SDRAM.
   * It is possible to store more than one image in linear way in the image-SDRAM. If ulImages is set greater
   * then one, the next correction image is used automatically by subsequent calls to GetImage().
   * ATTENTION: While this funtionality is enabled, the image SDRAM is exclusively
   * used and can not be accessed by other methods like ReadImageRAM(), WriteImageRAM(), etc.
   * \param bEnable enable/disable image correction
   * \param eType   defines type of image correction
   * \param ulStartAddr defines start address of correction-data in image SDRAM
   * \param ulSize number of bytes used as correction data; in general the size of the current image
   * \param ulImages number of correction images to use ( always >= 1)
   * \return
   *          0 ... on success \n
   */
  virtual int SetupImageCorrection(bool bEnable, eCorrectionType eType, unsigned long ulStartAddr, unsigned long ulSize, unsigned long ulImages) = 0;

  /** This method allows to store the raw sensor image in the FPGA attached image SDRAM 
   * during a GetImage() operation.
   * The length of the stored image corresponds to the current set image size.
   * The eType parameter would allow other settings than storing the RAW image, but this 
   * are reserved for future use. So only use SPLIT_RAW/SPLIT_OFF for this.
   * ATTENTION: While this funtionality is enabled, the image SDRAM is exclusively
   * used and can not be accessed by other methods like ReadImageRAM(), WriteImageRAM(), 
   * etc.
   * \param bType defines data splitting mode
   * \param ulDestAddr start address in the image SDRAM
   * \param ulSize ammount of data stored in image SDRAM (in general the size of the selected sensor window)
   * \return
   *          0 ... on success \n
   */
  virtual int SetupImageSplitting(eSplitType eType, unsigned long ulDestAddr, unsigned long ulSize) = 0;


  ///@}

};

//****************************************************************************
//                                                                OCameraTools
//****************************************************************************
/**
 *  Basic class for camera tools.
 */
class OREMOTECAMERA_EXP OCameraTools
{
 public:

  typedef enum
    {
      BAYER_PATTERN_GRGR = 1,    ///< bayer data start with green-red-green-red color pixels
      BAYER_PATTERN_BGBG = 2,    ///< bayer data start with blue-green-blue-green color pixels
      BAYER_PATTERN_RGRG = 3,    ///< bayer data start with red-green-red-green color pixels
      BAYER_PATTERN_GBGB = 4     ///< bayer data start with green-blue-green-blue color pixels
    }
  eBayerPattern;

 public:
  /** Transform a bayer encoded image into a rgb encoded image.
   * \param bayer source image buffer with bayer encoded image
   * \param rgb destination image buffer for rgb encoded image
   * \param width number of pixels in x direction
   * \param height number of pixels in y direction
   * \param ePattern specifies the bayer pixel order
   */
  static void Bayer2RGB(unsigned char *bayer, unsigned char *rgb,
                        int width, int height,
                        eBayerPattern ePattern = BAYER_PATTERN_GRGR);

  /** Transform a bayer encoded image into a white balanced rgb encoded image.
   *  This method performs two tasks in one step: \n
   *    a bayer to RGB conversion and white balance color correction. \n
   *  White balancing is done by multiplying the individual RGB channels
   *  with a given factor. White balancing is necessary to adjust for different
   *  light conditions (day light, bulbs, neon, ...) in a way, that
   *  white areas are reproduces as white independent of illumination.
   *  Normally white balancing should happen when a white/grey surface
   *  is shown to the camera. Grey is better because it better prevents
   *  from calibrating on saturated (0xff pixels) image areas.
   * \param bayer source image buffer with bayer encoded image
   * \param rgb destination image buffer for rgb encoded image
   * \param width number of pixels in x direction
   * \param height number of pixels in y direction
   * \param red multiplication factor (given in percent) for red color component
   * \param green multiplication factor (given in percent) for green color component
   * \param blue multiplication factor (given in percent) for blue color component
   * \param ePattern specifies the bayer pixel order (currently only GRGR and BGBG are supported)
   */
  static void Bayer2RGBWhiteBalance(unsigned char *bayer, unsigned char *rgb,
                                    int width, int height,
                                    int red, int green, int blue,
                                    eBayerPattern ePattern = BAYER_PATTERN_GRGR);

  /** Transform a bayer encoded image into a rgb encoded image using built in white balance.
   * This method performs a bayer to rgb conversion with white balancing
   * using the white balance factors stored in the camera's flash.
   * White balance factors are read from the flash using the method
   * GetWhiteBalanceFactors(). Afterwards Bayer2RGBWhiteBalance() is
   * called using this factors. If reading the factors from flash fails
   * the bayer to rgb conversion is still performed but no white balancing
   * is done and an error code is returned.
   * \param bayer source image buffer with bayer encoded image
   * \param rgb destination image buffer for rgb encoded image
   * \param width number of pixels in x direction
   * \param height number of pixels in y direction
   * \param ePattern specifies the bayer pixel order (currently only GRGR and BGBG are supported)
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h) \n
   *    -EAGAIN ... on incomplete or invalid configuration file /ffx/etc/wb.conf
   */
  static int Bayer2RGBBuiltIn(unsigned char *bayer, unsigned char *rgb,
                              int width, int height,
                              eBayerPattern ePattern = BAYER_PATTERN_GRGR);

  /** Read white balance factors stored in flash memory.
   * This method reads previously stored white balance factors from
   * flash memory (flash file system /ffx/etc/wb.conf). \n
   * White balance factors are percentage values, i.e. 100 means the
   * corresponding color component is left unchanged, whereas 115 means
   * the the corresponding color component is multiplied by the factor
   * 1.15. Values smaller than 100 lead to a reduction of the corresponding
   * color component.
   * \param red pointer to the red color component white balance factor
   * \param green pointer to the green color component white balance factor
   * \param blue pointer to the blue color component white balance factor
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h) \n
   *    -EAGAIN ... on incomplete or invalid configuration file /ffx/etc/wb.conf
   */
  static int GetWhiteBalanceFactors(int *red, int *green, int *blue);

  /** Store white balance factors to flash memory.
   * This method writes/stores white balance factors to
   * flash memory (flash file system /ffx/etc/wb.conf).
   * \param red red color component white balance factor
   * \param green green color component white balance factor
   * \param blue blue color component white balance factor
   * \return
   *          0 ... on success \n
   *     -errno ... on failure always negative (see errno.h)
   */
  static int StoreWhiteBalanceFactors(int red, int green, int blue);

  /** For Hardware(FPGA)-Whitebalancing lookup tables are used. This method reads the
   *  stored factors from flash memory (flash file system /ffx/etc/wb.conf) and
   *  builds up the whitebalancing tables that can directly be used with the
   *  OCamera::SetWhiteBalance() method.
   * \param WBtable 768 byte width array where the lookup-tables are created.
   * \return
   *          0 ... on success \n
   */
  static int SetupHardwareWhitebalancingFactors(unsigned char WBtable[768]);

};

//****************************************************************************
//                                                               OCameraInputs
//****************************************************************************
/**
 *  Basic class for access to the digital inputs of the camera.
 *  \par Example Usage
 *         -# create interface with poCreateInterface()
 *         -# open it with iOpen()
 *         -# work with the inputs using
 *             - iGetInputs()
 *             - iGetInput()
 *             - iSetTimeout()
 *             - iWaitForInput()
 *             - iWaitForInputs()
 *             - iWaitForInputChange()
 *             - iWaitForInputsChange()
 *          -# after work close interface with iClose()
 *          -# if interface is no longer needed call Destroy()
 */
class OREMOTECAMERA_EXP OCameraInputs
{
 public:

  /// digital camera inputs
  typedef enum
    {
      INPUT_I0    = 1,    ///< digital input nr 0
      INPUT_I1    = 2,    ///< digital input nr 1
      INPUT_I0_RE = 4,    ///< digital input nr 0 rising edge detect
      INPUT_I0_FE = 8,    ///< digital input nr 0 falling edge detect
      INPUT_I1_RE = 16,   ///< digital input nr 1 rising edge detect
      INPUT_I1_FE = 32,   ///< digital input nr 1 falling edge detect
      OVERLOAD    = 64,   ///< output overload signal (1=overload)

      INPUT_ALL = INPUT_I0 | INPUT_I1  ///< digital input 0 and 1
    }
  t_enCameraInput;

  // object creation/deletion
  /// Call this method to obtain a pointer to the inputs interface
  static OCameraInputs *poCreateInterface();
  /** Removes completely the inputs interface object created with poCreateInterface().
   * This should happen after iClose() was called and the interface is
   * no longer needed. \n
   * Be careful to <B>DO NOT USE THE INTERFACE POINTER</B> any longer after
   * having called this method!!
   */
  virtual void Destroy() = 0;

  /** Grants the access to the camera's digital inputs.
   *  Has to be called <B>PRIOR</B> to use any subsequent call
   *  to the OCameraInputs interface. \n
   *  When work with digital inputs is finished, the user should
   *  call iClose() for cleanup.
   * \return
   *        < 0 ... on failure, usage of digital inputs is not available
   */
  virtual int iOpen() = 0;
  /// Defined cleanup after the work with the digital inputs has been finished.
  virtual int iClose() = 0;

  /** Returns the state of all digital inputs and the state of
   *  the output overload signal.
   *  The overload is logic high whenever the total sum of output
   *  current on all digital outputs exceeds 1.8A. \n
   *  Use the OCameraInputs::t_enCameraInput bit masks to ask for the sate of
   *  individual digital inputs. \n
   *  This call never blocks.
   * \return
   *       >= 0 ... value of all digital inputs acc. to bit mask OCameraInputs::t_enCameraInput \n
   *       -errno ... on failure always negative (see errno.h)
   */
  virtual int iGetInputs() = 0;

  /** Returns the state (0 or 1) of one digital input.
   *  This call never blocks.
   * \param enIn bitmask for one digital input
   * \return
   *          0 ... digital input is logic low \n
   *          1 ... digital input is logic high \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iGetInput(t_enCameraInput enIn) = 0;

  /** Set the timeout to wait during an iWaitForInputxxx() operation.
   *  This may prevent from hanging if the input doesn't change. \n
   *  By default no timeout is used.
   * \param iTimeout timeout value in milliseconds (1E-3) \n
   *                 -1 ... no timeout is desired (infinite wait)
   * \return
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iSetTimeout(int iTimeout) = 0;

  /** Wait until the digital input pattern reaches a given state.
   *  The default behaviour is infinite blocking. To specify
   *  a defined timeout call iSetTimeout() before the first use of
   *  this function.
   * \param enIn digital input values, that should be observed
   * \param iState state of digital inputs to stop waiting
   * \return
   *     -errno ... always negative (see errno.h) if error or timeout occurred \n
   *     -ETIME ... timeout occurred (timer expired)
   */
  virtual int iWaitForInput(t_enCameraInput enIn, int iState) = 0;

  /** Wait until one, several or all inputs reach a given state.
   *  The program execution stops at this function until
   *   - timeout occurs OR
   *   - inputs according to <em>enIn</em> equal exact the corresponding bits in <em>iState</em>
   *
   *  The default behaviour is infinite blocking. To specify
   *  a defined timeout call iSetTimeout().
   * \param iMask set of OCameraInputs::t_enCameraInput values ORed together. \n
   *              For each input set in iMask the corresponding bit in iState denotes
   *              the desired state for this input to wait for.
   * \param iState state of digital inputs to stop waiting
   * \return
   *       >= 0 ... given condition is met for all inputs \n
   *     -errno ... always negative (see errno.h) if error or timeout occured
   */
  virtual int iWaitForInputs(int iMask, int iState) = 0;

  /** Wait until the specified digital input changes it's logic level.
   *  The default behaviour is infinite blocking. To specify
   *  a defined timeout call iSetTimeout().
   * \param enIn digital input value acc. to OCameraInputs::t_enCameraInput,
   *             that should be observed
   * \return
   *     -errno ... always negative (see errno.h) if error or timeout occured
   */
  virtual int iWaitForInputChange(t_enCameraInput enIn) = 0;

  /** Wait until one, several or all inputs change their logic levels.
   *  The call returns if <B>one</B> of the selected inputs change.
   *  The default behaviour is infinite blocking. To specify
   *  a defined timeout call iSetTimeout().
   * \param iMask set of OCameraInputs::t_enCameraInput values ORed together
   * \return
   *       >= 0 ... one of the selected inputs changed \n
   *     -errno ... always negative (see errno.h) if error or timeout occured
   */
  virtual int iWaitForInputsChange(int iMask) = 0;

  /** Edge detection bits are sticky, they remain in their state unless they are cleared by this method.
   *  To clear the state of an edge detection bit set the corresponding bit in iMask when calling iClearEdges.
   * \param iMask all input's bit position, that equal "1" will be cleared (only in case of edge detection inputs)
   * \return
   *          0 ... function completed successfully \n
   *     -errno ... always negative (see errno.h) if error or timeout occured
   */
  virtual int iClearEdges(int iMask) = 0;

  /** Retrive a file descriptor to perform select/poll on the input interface (e.g. wait until input has changed).
   * \return
   *       > 0 ... valid file descriptor\n
   */
  virtual int GetDriverFileDescriptor() = 0;
};

//****************************************************************************
//                                                              OCameraOutputs
//****************************************************************************
/**
 *  Basic class for access to the digital outputs of the camera.
 *  All outputs keep their states beyond destruction of the interface.
 *  \par Example Usage
 *         -# create interface with poCreateInterface()
 *         -# open it with iOpen()
 *         -# work with the outputs using
 *             - iGetOutputs()
 *             - iGetOutput()
 *             - iSetOutputs()
 *             - iSetOutput()
 *             - iToggleOutputs()
 *             - iToggleOutput()
 *          -# after work close interface with iClose()
 *          -# if interface is no longer needed call Destroy()
 */
class OREMOTECAMERA_EXP OCameraOutputs
{
 public:

  /// digital camera outputs
  typedef enum
    {
      OUTPUT_O0  = 1,    ///< digital output nr 0 (pink cable)
      OUTPUT_O1  = 2,    ///< digital output nr 1 (yellow cable)
      OUTPUT_O2  = 4,    ///< digital output nr 2 (red cable)
      OUTPUT_O3  = 8,    ///< digital output nr 3 (NOT USED)

      OUTPUT_OVERLOAD_UNLOCK = 16, ///< virtual output to unlock an overcurrent situation

      OUTPUT_ALL = OUTPUT_O0 | OUTPUT_O1 | OUTPUT_O2 | OUTPUT_O3  ///< all outputs (nr 0 to nr 3)
    }
  t_enCameraOutput;

  // object creation/deletion
  /** Call this method to obtain a pointer to the outputs interface.
   *  Does <I>NOT</I> affect the actual states of the outputs.
   */
  static OCameraOutputs *poCreateInterface();
  /** Removes completely the outputs interface object created with poCreateInterface().
   * This should happen after iClose() was called and the interface is
   * no longer needed.\n
   * Does <I>NOT</I> affect the actual states of the outputs. \n
   * Be careful to <B>DO NOT USE THE INTERFACE POINTER</B> any longer after
   * having called this method!!
   */
  virtual void Destroy() = 0;

  /** Grants the access to the camera's digital outputs.
   *  Has to be called <B>PRIOR</B> to use any subsequent call
   *  to the OCameraOutputs interface. \n
   *  When work with digital outputs is finished, the user should
   *  call iClose() for cleanup. \n
   *  Does <I>NOT</I> affect the actual states of the outputs.
   * \return
   *        < 0 ... on failure, usage of digital inputs is not available
   */
  virtual int iOpen() = 0;
  /** Defined cleanup after the work with the digital outputs has been finished.
   *  Does <I>NOT</I> affect the actual states of the outputs.
   */
  virtual int iClose() = 0;

  /** Returns the state of all digital outputs.
   *  Use the OCameraOutputs::t_enCameraOutput bitmasks to ask for the state of
   *  individual digital outputs. \n
   *  This call never blocks.
   * \return
   *       >= 0 ... value of all digital outputs acc. to OCameraOutputs::t_enCameraOutput \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iGetOutputs() = 0;

  /** Returns the state of one digital output.
   *  This call never blocks.
   * \param enOut bitmask for one digital output
   * \return
   *          0 ... digital output is logic low \n
   *          1 ... digital output is logic high \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iGetOutput(t_enCameraOutput enOut) = 0;

  /** Set one, several or all of the digital outputs.
   *  Only the digital outputs defined by iMask are set to the
   *  corresponding state of iState.
   * \param iMask bitmask generated by a bitwise OR of
   *              OCameraOutputs::t_enCameraOutput values
   * \param iState the corresponding state (0 or 1) on the same bit position
   *               as iMask
   * \return
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iSetOutputs(int iMask, int iState) = 0;

  /** Set one digital output to a defined state.
   * \param enOut number of output
   * \param iState set (1) or delete (0) one digital output
   * \return
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iSetOutput(t_enCameraOutput enOut, int iState) = 0;

  /** Toggle the current state of one, several or all digital outputs.
   * \param iMask bitmask generated by a bitwise OR of
   *              OCameraOutputs::t_enCameraOutput values
   * \return
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iToggleOutputs(int iMask) = 0;

  /** Toggle the current state of one digital outputs.
   * \param enOut number of output
   * \return
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iToggleOutput(t_enCameraOutput enOut) = 0;

};

//****************************************************************************
//                                                                 OCameraLEDs
//****************************************************************************
/**
 *  Basic class for access to the camera status LEDs on the cameras rear
 *  side. \n
 *  All LEDs keep their states beyond destruction of the interface.
 *  \par Example Usage
 *         -# create interface with poCreateInterface()
 *         -# open it with iOpen()
 *         -# work with the LEDs using
 *             - iGetLEDs()
 *             - iGetLED()
 *             - iSetLEDs()
 *             - iSetLED()
 *             - iToggleLEDs()
 *             - iToggleLED()
 *          -# after work close interface with iClose()
 *          -# if interface is no longer needed call Destroy()
 */
class OREMOTECAMERA_EXP OCameraLEDs
{
 public:

  /// camera LEDs
  typedef enum
    {
      LED_0_RED    = 0x01,    ///< LED nr A, red
      LED_0_GREEN  = 0x02,    ///< LED nr A, green
      LED_1_RED    = 0x04,    ///< LED nr B, red
      LED_1_GREEN  = 0x08,    ///< LED nr B, green
      LED_2_RED    = 0x10,    ///< LED nr C, red
      LED_2_YELLOW = 0x20,    ///< LED nr C, yellow
      LED_3_RED    = 0x40,    ///< LED nr D, red
      LED_3_YELLOW = 0x80,    ///< LED nr D, yellow

      LED_ALL_RED    = LED_0_RED    | LED_1_RED | LED_2_RED | LED_3_RED,  ///< all red LEDs
      LED_ALL_GREEN  = LED_0_GREEN  | LED_1_GREEN,                        ///< all green LEDs
      LED_ALL_YELLOW = LED_2_YELLOW | LED_3_YELLOW,                       ///< all yellow LEDs
      LED_ALL        = LED_ALL_RED  | LED_ALL_GREEN | LED_ALL_YELLOW      ///< all LEDs
    }
  t_enCameraLED;

  // object creation/deletion
  /** Call this method to obtain a pointer to the LEDs interface.
   *  Does <I>NOT</I> affect the actual states of the LEDs.
   */
  static OCameraLEDs *poCreateInterface();
  /** Removes completely the LEDs interface object created with poCreateInterface().
   * This should happen after iClose() was called and the interface is
   * no longer needed.\n
   * Does <I>NOT</I> affect the actual states of the LEDs.\n
   * Be careful to <B>DO NOT USE THE INTERFACE POINTER</B> any longer after
   * having called this method!!
   */
  virtual void Destroy() = 0;

  /** Grants the access to the camera's digital LEDs.
   *  Has to be called <B>PRIOR</B> to use any subsequent call
   *  to the OCameraLEDs interface. \n
   *  When work with LEDs is finished, the user should
   *  call iClose() for cleanup. \n
   *  Does <I>NOT</I> affect the actual states of the LEDs.
   * \return
   *        < 0 ... on failure, usage of LEDs is not available
   */
  virtual int iOpen() = 0;
  /** Defined cleanup after the work with the LEDs has been finished.
   *  Does <I>NOT</I> affect the actual states of the LEDs.
   */
  virtual int iClose() = 0;

  /** Returns the state of all LEDs.
   *  Use the OCameraLEDs::t_enCameraLED bitmasks to ask for the sate of
   *  individual LEDs. \n
   *  This call never blocks.
   * \return
   *       >= 0 ... value of all LEDs acc. to OCameraLEDs::t_enCameraLED \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iGetLEDs() = 0;

  /** Returns the state of one LED.
   *  This call never blocks.
   * \param enLED bitmask for one LED
   * \return
   *          0 ... LED is off \n
   *          1 ... LED is on \n
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iGetLED(t_enCameraLED enLED) = 0;

  /** Set one, several or all of the LEDs.
   * \param iMask bitmask generated by a bitwise OR of
   *              OCameraLEDs::t_enCameraLED values
   * \param iState the corresponding state (0 or 1) on the same bit position
   *               as iMask
   * \return
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iSetLEDs(int iMask, int iState) = 0;

  /** Set one LED to a defined state.
   * \param enLED number of LED
   * \param iState set (1) or delete (0) one LED
   * \return
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iSetLED(t_enCameraLED enLED, int iState) = 0;

  /** Toggle the current state of one, several or all camera LEDs.
   * \param iMask bitmask generated by a bitwise OR of
   *              OCameraLEDs::t_enCameraLED values
   * \return
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iToggleLEDs(int iMask) = 0;

  /** Toggle the current state of one LED.
   * \param enLED number of LED
   * \return
   *     -errno ... on failure always negative (see errno.h)
   */
  virtual int iToggleLED(t_enCameraLED enLED) = 0;
};

#endif // __OCAMERA_H__

OREMOTECAMERA_EXP OCamera* CreateCameraInterface( OCamera::eCameraType eCam, const char* url = "http://192.168.2.10:8080" );
OREMOTECAMERA_EXP OCameraInputs* CreateCameraInputs( const char* url = "http://192.168.2.10:8080" );
OREMOTECAMERA_EXP OCameraOutputs* CreateCameraOutputs( const char* url = "http://192.168.2.10:8080" );
OREMOTECAMERA_EXP OCameraLEDs* CreateCameraLEDs( const char* url = "http://192.168.2.10:8080" );

#endif
